@@ -25,6 +25,7 @@ SOFTWARE.
 import contextlib
 import os
 import sqlite3
+from typing import *
 import typing
 import disnake as discord
 import datetime
@@ -36,7 +37,7 @@ from dpys import utils
 RED = 0xD40C00
 BLUE = 0x0000FF
 GREEN = 0x32C12C
-version = "5.3.5"
+version = "5.4.1"
 EPHEMERAL = True
 warnings_db: aiosqlite.Connection
 muted_db: aiosqlite.Connection
@@ -46,13 +47,18 @@ curse_db: aiosqlite.Connection
 print("We recommend that you read https://jgltechnologies.com/dpys before you use DPYS.")
 
 
-async def setup(dir: str):
+async def setup(bot: commands.BotBase, dir: str):
     global warnings_db, muted_db, rr_db, curse_db
     warnings_db = await aiosqlite.connect(os.path.join(dir, "warnings.db"))
     muted_db = await aiosqlite.connect(os.path.join(dir, "muted.db"))
     rr_db = await aiosqlite.connect(os.path.join(dir, "rr.db"))
     curse_db = await aiosqlite.connect(os.path.join(dir, "curse.db"))
 
+    bot.warnings_db = warnings_db
+    bot.muted_db = muted_db
+    bot.rr_db = rr_db
+    bot.curse_db = curse_db
+
 
 class misc:
 
@@ -99,7 +105,7 @@ class admin:
 
     @staticmethod
     async def mute(inter: ApplicationCommandInteraction, member: discord.Member, role_add: int,
-                   role_remove: typing.Optional[int] = None, reason: str = None) -> None:
+                   role_remove: typing.Optional[int] = None, reason: str = None, msg: str = None) -> None:
         if inter.guild.get_role(role_add) in member.roles:
             await inter.response.send_message(f"{member.name}#{member.discriminator} is already muted.",
                                               ephemeral=EPHEMERAL)
@@ -114,78 +120,82 @@ class admin:
                 with contextlib.suppress(discord.Forbidden, discord.HTTPException):
                     await member.remove_roles(inter.guild.get_role(role_remove))
             if reason is None:
-                await inter.response.send_message(f"Muted {str(member)}.", ephemeral=EPHEMERAL)
+                await inter.response.send_message(msg or f"Muted {str(member)}.", ephemeral=EPHEMERAL)
             else:
-                await inter.response.send_message(f"Muted {member.name}#{member.discriminator}. Reason: {reason}",
-                                                  ephemeral=EPHEMERAL)
+                await inter.response.send_message(
+                    msg or f"Muted {member.name}#{member.discriminator}. Reason: {reason}",
+                    ephemeral=EPHEMERAL)
 
     @staticmethod
     async def unmute(inter: ApplicationCommandInteraction, member: discord.Member, role_remove: int,
-                     role_add: typing.Optional[int] = None) -> None:
+                     role_add: typing.Optional[int] = None, msg: str = None) -> bool:
         if inter.guild.get_role(role_remove) not in member.roles:
             await inter.response.send_message(f"{member.name}#{member.discriminator} is not muted.",
                                               ephemeral=EPHEMERAL)
-            return
+            return False
         else:
             if not isinstance(inter.guild.get_role(role_remove), discord.Role):
-                return
+                return False
             await member.remove_roles(inter.guild.get_role(role_remove))
             if role_add is not None:
                 with contextlib.suppress(discord.Forbidden, discord.HTTPException):
                     await member.add_roles(inter.guild.get_role(role_add))
-            await inter.response.send_message(f"Unmuted {member.name}#{member.discriminator}.", ephemeral=EPHEMERAL)
+            await inter.response.send_message(msg or f"Unmuted {member.name}#{member.discriminator}.",
+                                              ephemeral=EPHEMERAL)
+            return True
 
     @staticmethod
-    async def clear(inter: ApplicationCommandInteraction, amount: typing.Optional[int] = 99999999999999999) -> int:
+    async def clear(inter: ApplicationCommandInteraction, amount: typing.Optional[int] = 99999999999999999,
+                    msg: str = None) -> int:
         limit = datetime.datetime.now() - datetime.timedelta(weeks=2)
         purged = await inter.channel.purge(limit=amount, after=limit)
         purged = len(purged)
         if purged != 1:
-            message = f"Cleared {purged} messages."
+            message = msg or f"Cleared {purged} messages."
         else:
-            message = f"Cleared {purged} message."
+            message = msg or f"Cleared {purged} message."
         await inter.response.send_message(message, ephemeral=EPHEMERAL)
         return purged
 
     @staticmethod
     async def kick(inter: ApplicationCommandInteraction, member: discord.Member,
-                   reason: typing.Optional[str] = None) -> None:
+                   reason: typing.Optional[str] = None, msg: str = None) -> None:
         if len(str(reason)) > 256:
             reason = reason[:256]
         await member.kick(reason=reason)
         if reason is None:
-            message = f"Kicked {member.name}#{member.discriminator}."
+            message = msg or f"Kicked {member.name}#{member.discriminator}."
         else:
-            message = f"Kicked {member.name}#{member.discriminator}. Reason: {reason}"
+            message = msg or f"Kicked {member.name}#{member.discriminator}. Reason: {reason}"
         await inter.response.send_message(message, ephemeral=EPHEMERAL)
 
     @staticmethod
     async def ban(inter: ApplicationCommandInteraction, member: discord.Member,
-                  reason: typing.Optional[str] = None) -> None:
+                  reason: typing.Optional[str] = None, msg: str = None) -> None:
         if len(str(reason)) > 256:
             reason = reason[:256]
         await member.ban(reason=reason)
         if reason is None:
-            message = f"Banned {member.name}#{member.discriminator}."
+            message = msg or f"Banned {member.name}#{member.discriminator}."
         else:
-            message = f"Banned {member.name}#{member.discriminator}. Reason: {reason}"
+            message = msg or f"Banned {member.name}#{member.discriminator}. Reason: {reason}"
         await inter.response.send_message(message, ephemeral=EPHEMERAL)
 
     @staticmethod
     async def softban(inter: ApplicationCommandInteraction, member: discord.Member,
-                      reason: typing.Optional[str] = None) -> None:
+                      reason: typing.Optional[str] = None, msg: str = None) -> None:
         if len(str(reason)) > 256:
             reason = reason[:256]
         await member.ban(reason=reason)
         if reason is None:
-            message = f"Soft banned {member.name}#{member.discriminator}."
+            message = msg or f"Soft banned {member.name}#{member.discriminator}."
         else:
-            message = f"Soft banned {member.name}#{member.discriminator}. Reason: {reason}"
+            message = msg or f"Soft banned {member.name}#{member.discriminator}. Reason: {reason}"
         await inter.response.send_message(message, ephemeral=EPHEMERAL)
         await member.unban()
 
     @staticmethod
-    async def unban(inter: ApplicationCommandInteraction, member: typing.Union[str, int]) -> None:
+    async def unban(inter: ApplicationCommandInteraction, member: typing.Union[str, int], msg: str = None) -> bool:
         bans = await inter.guild.bans()
         if isinstance(member, int):
             ban = [ban for ban in bans if ban.user.id == member]
@@ -198,9 +208,10 @@ class admin:
                    discrim and ban.user.name == name]
         if not ban:
             await inter.response.send_message(f"{member} is not banned.", ephemeral=EPHEMERAL)
-            return
+            return False
         await inter.guild.unban(ban[0].user)
-        await inter.response.send_message(f"Unbanned {member}.", ephemeral=EPHEMERAL)
+        await inter.response.send_message(msg or f"Unbanned {member}.", ephemeral=EPHEMERAL)
+        return True
 
 
 class curse:
@@ -468,7 +479,7 @@ class warnings:
             await inter.response.send_message(f"{user.name}#{user.discriminator} has no warnings.", ephemeral=EPHEMERAL)
 
     @staticmethod
-    async def unwarn(inter: ApplicationCommandInteraction, member, number: typing.Union[int, str]) -> None:
+    async def unwarn(inter: ApplicationCommandInteraction, member, number: typing.Union[int, str]) -> bool:
         user = member
         guild = str(inter.guild.id)
         member = str(member.id)
@@ -479,22 +490,22 @@ class warnings:
             async with db.execute("SELECT reason FROM warnings WHERE guild = ? and member_id = ?",
                                   (guild, member)) as cursor:
                 count = 0
-                async for entry in cursor:
+                async for _ in cursor:
                     count += 1
         except:
             msg = f"{user.name}#{user.discriminator} has no warnings."
             await inter.response.send_message(msg, ephemeral=EPHEMERAL)
-            return
+            return False
         if count < 1:
             msg = f"{user.name}#{user.discriminator} has no warnings."
             await inter.response.send_message(msg, ephemeral=EPHEMERAL)
-            return
+            return False
         if number == "all":
             await db.execute("DELETE FROM warnings WHERE guild = ? and member_id = ?", (guild, member))
             await db.commit()
             msg = f"Cleared all warnings from {user.name}#{user.discriminator}."
             await inter.response.send_message(msg, ephemeral=EPHEMERAL)
-            return
+            return True
         else:
             try:
                 if "," in number:
@@ -531,17 +542,20 @@ class warnings:
                     await db.commit()
                     msg = f"Cleared {user.name}#{user.discriminator}'s #{number} warning."
                     await inter.response.send_message(msg, ephemeral=EPHEMERAL)
+                    return True
             except:
                 if number == "all":
                     msg = f"{user.name}#{user.discriminator} has no warnings."
                 else:
                     msg = f"{user.name}#{user.discriminator} does not have that many warnings."
                 await inter.response.send_message(msg, ephemeral=EPHEMERAL)
+                return False
 
     @staticmethod
     async def punish(inter: ApplicationCommandInteraction, member: discord.Member,
                      punishments: typing.Mapping[int, Punishment],
-                     add_role: typing.Optional[int] = None, remove_role: typing.Optional[int] = None) -> None:
+                     add_role: typing.Optional[int] = None, remove_role: typing.Optional[int] = None,
+                     before: Optional[Callable[[int, Punishment, discord.Member], Awaitable[None]]] = None) -> None:
         memberid = str(member.id)
         guild = str(inter.guild.id)
         db = warnings_db
@@ -574,6 +588,7 @@ class warnings:
                 await db.execute("INSERT INTO tempban (guild,member,time) VALUES (?,?,?)",
                                  (guild, memberid, time))
                 await db.commit()
+                await before(warnings_number, punishments[warnings_number], member)
                 return
             else:
                 add_role = inter.guild.get_role(add_role)
@@ -591,16 +606,18 @@ class warnings:
                             await member.remove_roles(remove_role)
                     await mute_on_join.mute_add(inter.guild, member)
                     time = datetime.datetime.now() + datetime.timedelta(seconds=time)
-                    with contextlib.suppress(sqlite3.Error):
-                        await db.execute("INSERT INTO tempmute (guild,member,time) VALUES (?,?,?)",
-                                         (guild, memberid, time))
-                        await db.commit()
+                    await db.execute("INSERT INTO tempmute (guild,member,time) VALUES (?,?,?)",
+                                     (guild, memberid, time))
+                    await db.commit()
+                    await before(warnings_number, punishments[warnings_number], member)
         else:
             if punishments[warnings_number].punishment == "ban":
                 await member.ban(reason=f"You have received {warnings_number} warnings.")
+                await before(warnings_number, punishments[warnings_number], member)
                 return
             if punishments[warnings_number].punishment == "kick":
                 await member.kick(reason=f"You have received {warnings_number} warnings.")
+                await before(warnings_number, punishments[warnings_number], member)
                 return
             if punishments[warnings_number].punishment == "mute":
                 add_role = inter.guild.get_role(add_role)
@@ -612,10 +629,11 @@ class warnings:
                 if add_role is not None:
                     await member.add_roles(add_role)
                 await mute_on_join.mute_add(inter.guild, member)
+                await before(warnings_number, punishments[warnings_number], member)
 
     @staticmethod
-    async def temp_mute_loop(bot: commands.Bot, add_role_func: typing.Awaitable,
-                             remove_role_func: typing.Optional[typing.Awaitable] = None) -> None:
+    async def temp_mute_loop(bot: commands.Bot, add_role_func: Callable[[int], Awaitable[Optional[int]]],
+                             remove_role_func: Optional[Callable[[int], Awaitable[Optional[int]]]] = None) -> None:
         db = warnings_db
         with contextlib.suppress(sqlite3.Error):
             async with db.execute("SELECT guild,member,time FROM tempmute") as cursor:
@@ -639,9 +657,9 @@ class warnings:
                     else:
                         role_remove = await remove_role_func(int(guild_id))
                     if datetime.datetime.now() >= time:
+                        with contextlib.suppress(discord.Forbidden, discord.HTTPException):
+                            await member.add_roles(guild.get_role(int(role_remove)))
                         if role_add is not None:
-                            with contextlib.suppress(discord.Forbidden, discord.HTTPException):
-                                await member.add_roles(guild.get_role(int(role_remove)))
                             with contextlib.suppress(discord.Forbidden, discord.HTTPException):
                                 await member.remove_roles(guild.get_role(int(role_add)))
                         with contextlib.suppress(sqlite3.Error):
